#include <linux/pci.h>
#include <linux/delay.h>

#include "et131x.h"
#include "et131x_dma.h"
#include "et131x_registers.h"

void et131x_txdma_disable(struct et131x *et131x) {
	et131x->regs->txdma.ctrl.value = 0x101;
}

void et131x_txdma_enable(struct et131x *et131x) {
	union et131x_txdma_ctrl ctrl = {{0}};
	ctrl.sngl_epkt_mode = 1;
	ctrl.halt = 0;
	ctrl.cache_thrshld = 0;
	et131x->regs->txdma.ctrl = ctrl;
}

int et131x_txdma_init(struct et131x *et131x) {
	int err = 0;

	// Create a single packet tx buffer
	
	et131x->tx.desc = pci_alloc_consistent(et131x->pcidev, sizeof *et131x->tx.desc, &et131x->tx.desc_dma);
	if(!et131x->tx.desc) {
		printk(KERN_ERR "et131x: pci_alloc_consistent() failed\n");
		err = -EIO;
		goto exit;
	}

	et131x->tx.status = pci_alloc_consistent(et131x->pcidev, sizeof *et131x->tx.status, &et131x->tx.status_dma);
	if(!et131x->tx.status) {
		printk(KERN_ERR "et131x: pci_alloc_consistent() failed\n");
		err = -EIO;
		goto exit_free_desc;
	}

	et131x->regs->txdma.pr_base_lo = et131x->tx.desc_dma;
	et131x->regs->txdma.pr_base_hi = et131x->tx.desc_dma >> 32;
	et131x->regs->txdma.pr_num_des = 0;

	et131x->regs->txdma.wb_addr_lo = et131x->tx.status_dma;
	et131x->regs->txdma.wb_addr_hi = et131x->tx.status_dma >> 32;

//exit_free_status:
	pci_free_consistent(et131x->pcidev, sizeof *et131x->tx.status, et131x->tx.status, et131x->tx.status_dma);
exit_free_desc:
	pci_free_consistent(et131x->pcidev, sizeof *et131x->tx.desc, et131x->tx.desc, et131x->tx.desc_dma);
exit:
	return err;
}

void et131x_txdma_exit(struct et131x *et131x) {
	pci_free_consistent(et131x->pcidev, sizeof *et131x->tx.status, et131x->tx.status, et131x->tx.status_dma);
	pci_free_consistent(et131x->pcidev, sizeof *et131x->tx.desc, et131x->tx.desc, et131x->tx.desc_dma);
}

void et131x_txdma_send(struct et131x* et131x, void *buf, size_t len) {
	dma_addr_t buf_dma;

	buf_dma = pci_map_single(et131x->pcidev, buf, len, PCI_DMA_TODEVICE);
	et131x->tx.desc[0].data_lo = buf_dma;
	et131x->tx.desc[0].data_hi = buf_dma >> 32;
	et131x->tx.desc[0].length = len;
	et131x->tx.desc[0].vlan.value = 0;
	et131x->tx.desc[0].flags.value = 0x3;
	wmb();
	et131x->regs->txdma.service_request.addr = 0;
	mdelay(1);
	pci_unmap_single(et131x->pcidev, buf_dma, len, PCI_DMA_TODEVICE);
}
