#ifndef ET131X_H
#define ET131X_H

#include <linux/netdevice.h>
#include <linux/mii.h>

#include "et131x_registers.h"

#define ET1310_PCI_PM_CAPABILITY    0x40
#define ET1310_PCI_PM_CSR           0x44
#define ET1310_PCI_MAX_PYLD         0x4C
#define ET1310_PCI_DEV_CTRL         0x50
#define ET1310_PCI_DEV_STAT         0x52
#define ET1310_NMI_DISABLE          0x61
#define ET1310_PCI_MAC_ADDRESS      0xA4
#define ET1310_PCI_EEPROM_STATUS    0xB2
#define ET1310_PCI_PHY_INDEX_REG    0xB4
#define ET1310_PCI_ACK_NACK         0xC0
#define ET1310_PCI_REPLAY           0xC2
#define ET1310_PCI_L0L1LATENCY      0xCF
#define ET1310_PCI_SEL_PHY_CTRL     0xE4
#define ET1310_PCI_ADVANCED_ERR     0x100

struct et131x {
	struct pci_dev *pcidev;
	struct net_device *netdev;
	struct et131x_registers *regs;
	int using_dac;
	struct net_device_stats stats;
	struct mii_if_info mii;
	int xcvr_addr;
	uint32_t xcvr_id;
	
	struct {
		struct et131x_rx_desc *desc;
		dma_addr_t desc_dma;
		struct et131x_rx_status *status;
		dma_addr_t status_dma;
		char *buf;
		dma_addr_t buf_dma;
		size_t bufsize;
	} rx;

	struct {
		struct et131x_tx_desc *desc;
		dma_addr_t desc_dma;
		union et131x_tx_status *status;
		dma_addr_t status_dma;
		char *buf;
		dma_addr_t buf_dma;
		size_t bufsize;
	} tx;
};

extern int et131x_net_open(struct net_device *netdev);
extern int et131x_net_close(struct net_device *netdev);
extern int et131x_net_xmit(struct sk_buff *skb, struct net_device *netdev);
extern struct net_device_stats *et131x_net_stats(struct net_device *netdev);


extern void et131x_wake(struct et131x *et131x);
extern void et131x_soft_reset(struct et131x *et131x);
extern void et131x_disable_interrupts(struct et131x *et131x);

#endif
