#ifndef ET131X_DMA_H
#define ET131X_DMA_H

#include "et131x.h"

// Send buffer descriptor

struct et131x_tx_desc {
	uint32_t data_hi;
	uint32_t data_lo;
	uint16_t length;
	union et131x_tx_desc_vlan {
		struct {
			uint16_t tag:12;
			uint16_t cfi:1;
			uint16_t prio:3;
		};
		uint16_t value;
	} vlan;
	union et131x_tx_desc_flags {
		struct {
			uint32_t last:1;
			uint32_t first:1;
			uint32_t ir:1;
			uint32_t cw:1;
			uint32_t bp:1;
			uint32_t pf:1;
			uint32_t error:1;
			uint32_t crc:1;
			uint32_t mac:1;
			uint32_t pad:1;
			uint32_t huge:1;
			uint32_t vlan:1;
			uint32_t ipa:1;
			uint32_t tcpa:1;
			uint32_t udpa:1;
			uint32_t unused:17;
		};
		uint32_t value;
	} flags;
};

union et131x_tx_status {
	struct {
		uint32_t serv_cpl:10;
		uint32_t serv_cpl_wrap:1;
		uint32_t unused:21;
	};
	uint32_t value;
};

// Receive buffer descriptor

struct et131x_rx_desc {	
	uint32_t data_lo;
	uint32_t data_hi;
	uint32_t index; // 10 bits
};

struct et131x_rx_status {
	union et131x_rx_status_flags {
		struct {
			uint16_t hp:1; // hash pass
			uint16_t ipa:1; // IP checksum assist
			uint16_t ipp:1; // IP checksum pass
			uint16_t tcpa:1; // TCP checksum assist
			uint16_t tcpp:1; // TCP checksum pass
			uint16_t wol:1;	// WOL Event
			uint16_t rxmac_error:1;
			uint16_t drop:1;
			uint16_t ft:1;	// Frame Truncated
			uint16_t jp:1;	// Jumbo Packet
			uint16_t vp:1;	// VLAN Packet
			uint16_t unused:5;
		};
		uint16_t value;
	} flags;

	union et131x_rx_status_asw {
		struct {
			uint16_t prev_pkt_dropped:1; // e.g. IFG too small on previous
			uint16_t rx_dv_event:1; // short receive event detected
			uint16_t false_carrier_event:1; // bad carrier since last good packet
			uint16_t code_err:1; // one or more nibbles signalled as errors
			uint16_t crc_err:1; // CRC error
			uint16_t len_chk_err:1; // frame length field incorrect
			uint16_t too_long:1; // frame length > 1518 bytes
			uint16_t ok:1; // valid CRC + no code error
			uint16_t multicast:1; // has a multicast address
			uint16_t broadcast:1; // has a broadcast address
			uint16_t dribble_nibble:1; // spurious bits after EOP
			uint16_t control_frame:1; // is a control frame
			uint16_t pause_frame:1; // is a pause frame
			uint16_t unsupported_op:1; // unsupported OP code
			uint16_t vlan_tag:1; // VLAN tag detected
			uint16_t long_evt:1; // Rx long event
		};
		uint16_t value;
	} asw;

	uint16_t length;

	union et131x_rx_status_index {
		struct {
			uint16_t buffer:10;
			uint16_t ring:2;
			uint16_t unused:4;
		};
		uint16_t value;
	} index;
			
};

extern void et131x_txdma_disable(struct et131x *et131x);
extern void et131x_txdma_enable(struct et131x *et131x);
extern int et131x_txdma_init(struct et131x *et131x);
extern void et131x_txdma_exit(struct et131x *et131x);
extern void et131x_txdma_send(struct et131x *et131x, void *buf, size_t len);

#endif
