#ifndef ET131X_PHY_REGISTERS_H
#define ET131X_PHY_REGISTERS_H

#define ET131X_MII_ESR         0x0f
#define ET131X_MII_INDEX       0x10
#define ET131X_MII_DATA        0x11
#define ET131X_MII_MCR         0x12
#define ET131X_MII_LCR         0x13
#define ET131X_MII_MICR        0x15
#define ET131X_MII_PHY_CONFIG  0x16
#define ET131X_MII_PHY_CONTROL 0x17
#define ET131X_MII_IMR         0x18
#define ET131X_MII_ISR         0x19
#define ET131X_MII_PSR         0x1a
#define ET131X_MII_LCR1        0x1b
#define ET131X_MII_LCR2        0x1c

union et131x_mii_bmcr {
	struct {
		uint16_t res1:6;
		uint16_t speed_1000_sel:1;
		uint16_t col_test:1;
		uint16_t duplex_mode:1;
		uint16_t restart_autoneg:1;
		uint16_t isolate:1;
		uint16_t power_down:1;
		uint16_t enable_autoneg:1;
		uint16_t speed_sel:1;
		uint16_t loopback:1;
		uint16_t reset:1;
	};
	uint16_t value;
};

union et131x_mii_bmsr {
	struct {
		uint16_t ext_cap:1;
		uint16_t jabber_detect:1;
		uint16_t link_status:1;
		uint16_t auto_neg_able:1;
		uint16_t remote_fault:1;
		uint16_t auto_neg_complete:1;
		uint16_t preamble_supress:1;
		uint16_t res1:1;
		uint16_t extend_status:1;
		uint16_t link_100T2hdx:1;
		uint16_t link_100T2fdx:1;
		uint16_t link_10hdx:1;
		uint16_t link_10fdx:1;
		uint16_t link_100hdx:1;
		uint16_t link_100fdx:1;
		uint16_t link_100T4:1;
	};
	uint16_t value;
};

union et131x_mii_idr1 {
	struct {
		uint16_t ieee_address:16;
	};
	uint16_t value;
};

union et131x_mii_idr2 {
	struct {
		uint16_t rev_no:4;
		uint16_t model_no:6;
		uint16_t ieee_address:6;
	};
	uint16_t value;
};

union et131x_mii_anar {
	struct {
		uint16_t selector:5;
		uint16_t cap_10hdx:1;
		uint16_t cap_10fdx:1;
		uint16_t cap_100hdx:1;
		uint16_t cap_100fdx:1;
		uint16_t cap_100T4:1;
		uint16_t cap_pause:1;
		uint16_t cap_asmpause:1;
		uint16_t res1:1;
		uint16_t remote_fault:1;
		uint16_t res2:1;
		uint16_t np_indication:1;
	};
	uint16_t value;
};

union et131x_mii_anlpar {
	struct {
		uint16_t selector:5;
		uint16_t cap_10hdx:1;
		uint16_t cap_10fdx:1;
		uint16_t cap_100hdx:1;
		uint16_t cap_100fdx:1;
		uint16_t cap_100T4:1;
		uint16_t cap_pause:1;
		uint16_t cap_asmpause:1;
		uint16_t res1:1;
		uint16_t remote_fault:1;
		uint16_t acknowledge:1;
		uint16_t np_indication:1;
	};
	uint16_t value;
};

union et131x_mii_aner {
	struct {
		uint16_t lp_an_able:1;
		uint16_t page_rx:1;
		uint16_t np_able:1;
		uint16_t lp_np_able:1;
		uint16_t pdf:1;
		uint16_t res:11;
	};
	uint16_t value;
};

union et131x_mii_annptr {
	struct {
		uint16_t msg:11;
		uint16_t toggle:1;
		uint16_t ack2:1;
		uint16_t msg_page:1;
		uint16_t res1:1;
		uint16_t np:1;
	};
	uint16_t value;
};

union et131x_mii_lpnpr {
	struct {
		uint16_t msg:11;
		uint16_t toggle:1;
		uint16_t ack2:1;
		uint16_t msg_page:1;
		uint16_t ack:1;
		uint16_t np:1;
	};
	uint16_t value;
};

union et131x_mii_gcr {
	struct {
		uint16_t res:8;
		uint16_t link_1000hdx:1;
		uint16_t link_1000fdx:1;
		uint16_t port_type:1;
		uint16_t ms_value:1;
		uint16_t ms_config_en:1;
		uint16_t test_mode:3;
	};
	uint16_t value;
};

union et131x_mii_gsr {
	struct {
		uint16_t idle_err_cnt:8;
		uint16_t res:2;
		uint16_t link_1000hdx:1;
		uint16_t link_1000fdx:1;
		uint16_t remote_rx_status:1;
		uint16_t local_rx_status:1;
		uint16_t ms_resolve:1;
		uint16_t ms_config_fault:1;
	};
	uint16_t value;
};

union et131x_mii_esr {
	struct {
		uint16_t res:12;
		uint16_t link_1000hdx:1;
		uint16_t link_1000fdx:1;
		uint16_t link_1000Xhdx:1;
		uint16_t link_1000Xfdx:1;
	};
	uint16_t value;
};

union et131x_mii_lcr {
	struct {
		uint16_t res:10;
		uint16_t line_driver_en:1;
		uint16_t replica_en:1;
		uint16_t all_digital_en:1;
		uint16_t pmd_en:1;
		uint16_t pcs_en:1;
		uint16_t mii_en:1;
	};
	uint16_t value;
};


union et131x_mii_micr {
	struct {
		uint16_t preamble_supress_en:1;
		uint16_t res3:1;
		uint16_t ignore_10g_fr:1;
		uint16_t res2:1;
		uint16_t mi_error_count:7;
		uint16_t res1:5;
	};
	uint16_t value;
};

union et131x_mii_phy_config {
	struct {
		uint16_t mac_if_mode:3;
		uint16_t res2:1;
		uint16_t sys_clock_en:1;
		uint16_t tx_clock_en:1;
		uint16_t group_mdio_en:1;
		uint16_t alternate_np:1;
		uint16_t tbi_rate:1;
		uint16_t pbi_detect:1;
		uint16_t speed_downshift:2;
		uint16_t tx_fifo_depth:2;
		uint16_t res1:1;
		uint16_t crs_tx_en:1;
	};
	uint16_t value;
};

union et131x_mii_phy_control {
	struct {
		uint16_t force_int:1;
		uint16_t res4:1;
		uint16_t preamble_gen_en:1;
		uint16_t tp_loopback_10baseT:1;
		uint16_t sqe_10baseT:1;
		uint16_t jabber_10baseT:1;
		uint16_t res3:5;
		uint16_t downshift_attempts:2;
		uint16_t res2:1;
		uint16_t tdr_en:1;
		uint16_t res1:1;
	};
	uint16_t value;
};

union et131x_mii_imr {
	struct {
		uint16_t int_en:1;
		uint16_t automatic_speed:1;
		uint16_t link_status:1;
		uint16_t rx_status:1;
		uint16_t fifo_over_underflow:1;
		uint16_t err_counter_full:1;
		uint16_t np_rx:1;
		uint16_t hi_bit_err:1;
		uint16_t autoneg_status:1;
		uint16_t mdio_sync_lost:1;
		uint16_t res1:6;
	};
	uint16_t value;
};

union et131x_mii_isr {
	struct {
		uint16_t int_en:1;
		uint16_t automatic_speed:1;
		uint16_t link_status:1;
		uint16_t rx_status:1;
		uint16_t fifo_over_underflow:1;
		uint16_t err_counter_full:1;
		uint16_t np_rx:1;
		uint16_t hi_bit_err:1;
		uint16_t autoneg_status:1;
		uint16_t mdio_sync_lost:1;
		uint16_t res1:6;
	};
	uint16_t value;
};

union et131x_mii_psr {
	struct {
		uint16_t asymmetric_dir:1;
		uint16_t pause_en:1;
		uint16_t autoneg_en:1;
		uint16_t collision_status:1;
		uint16_t rx_status:1;
		uint16_t tx_status:1;
		uint16_t link_status:1;
		uint16_t duplex_status:1;
		uint16_t speed_status:2;
		uint16_t polarity_status:1;
		uint16_t mdi_x_status:1;
		uint16_t autoneg_status:1;
		uint16_t autoneg_fault:2;
		uint16_t res1:1;
	};
	uint16_t value;
};

union et131x_mii_lcr1 {
	struct {
		uint16_t pulse_stretch0:1;
		uint16_t pulse_stretch1:1;
		uint16_t pulse_dur:2;
		uint16_t res3:2;
		uint16_t res2:2;
		uint16_t led_collision:2;
		uint16_t led_10baseT:2;
		uint16_t led_dup_indicate:2;
		uint16_t res1:2;
	};
	uint16_t value;
};

union et131x_mii_lcr2 {
	struct {
		uint16_t led_1000BaseT:4;
		uint16_t led_100BaseTX:4;
		uint16_t led_tx_rx:4;
		uint16_t led_link:4;
	};
	uint16_t value;
};

#endif
