#ifndef ET131X_REGISTERS_H
#define ET131X_REGISTERS_H

/* ET131x register map */

struct et131x_registers_global {
	// 0x0000 tx/rx queue start/end addresses (10 bits)

	uint32_t txq_start_addr;
	uint32_t txq_end_addr;
	uint32_t rxq_start_addr;
	uint32_t rxq_end_addr;

	// 0x0010 power management control status register

	union et131x_pm_ctrl {
		struct {
			uint32_t gigephy:1;
			uint32_t jagcore_tx:1;
			uint32_t jagcore_rx:1;
			uint32_t sysclk_gate:1;
			uint32_t txclk_gate:1;
			uint32_t rxclk_gate:1;
			uint32_t phy_sw_coma:1;
			uint32_t phy_lped:1;
			uint32_t jagcore_tx_rdy:1;
			uint32_t jagcore_rx_rdy:1;
			uint32_t unused:22;
		};
		uint32_t value;
	} pm_ctrl;

	// 0x0014 power management state

	union et131x_pm_state {
		struct {
			uint32_t cs:2;
			uint32_t unused:30;
		};
		uint32_t value;
	} pm_state;

	// 0x0018 interrupt status

	union et131x_int {
		struct {
			uint32_t unused0:1;
			uint32_t unused1:1;
			uint32_t unused2:1;
			uint32_t txdma_isr:1;
			uint32_t txdma_err:1;
			uint32_t rxdma_xfr_done:1;
			uint32_t rxdma_fb_ring0_low:1;
			uint32_t rxdma_fb_ring1_low:1;
			uint32_t rxdma_pkt_stat_ring_low:1;
			uint32_t rxdma_err:1;
			uint32_t unused10:1;
			uint32_t unused11:1;
			uint32_t unused12:1;
			uint32_t unused13:1;
			uint32_t watchdog:1;
			uint32_t wake_on_lan:1;
			uint32_t phy:1;
			uint32_t txmac:1;
			uint32_t rxmac:1;
			uint32_t mac_stat:1;
			uint32_t slv_timeout:1;
			uint32_t unused:11;
		};
		uint32_t value;
	} int_status;

	// 0x001c interrupt mask

	union et131x_int int_mask;

	// 0x0020 interrupt alias clear mask enable

	union et131x_int int_alias_clr_en;

	// 0x0024 interrupt status alias

	union et131x_int int_status_alias;

	// 0x0028 software reset

	union et131x_sw_reset {
		struct {
			uint32_t txdma:1;
			uint32_t rxdma:1;
			uint32_t txmac:1;
			uint32_t rxmac:1;
			uint32_t mac:1;
			uint32_t mac_stat:1;
			uint32_t mmc:1;
			uint32_t unused:24;
			uint32_t selfclr_disable:1;
		};
		uint32_t value;
	} sw_reset;

	// 0x002c SLV timer (24 bits)

	uint32_t slv_timer;

	// 0x0030 MSI configuration

	union et131x_msi_config {
		struct {
			uint16_t vector; // 5 bits
			uint16_t tc; // 3 bits
		};
		uint32_t value;
	} msi_config;

	// 0x0034 loopback

	union et131x_loopback {
		struct {
			uint32_t mac:1;
			uint32_t dma:1;
			uint32_t unused:30;
		};
		uint32_t value;
	} loopback;

	uint32_t watchdog_timer;
};

union et131x_wrapaddr4 {
	struct {
		uint32_t addr:4;
		uint32_t wrap:1;
		uint32_t unused:27;
	};
	uint32_t value;
};

union et131x_wrapaddr10 {
	struct {
		uint32_t addr:10;
		uint32_t wrap:1;
		uint32_t unused:21;
	};
	uint32_t value;
};

union et131x_wrapaddr12 {
	struct {
		uint32_t addr:12;
		uint32_t wrap:1;
		uint32_t unused:19;
	};
	uint32_t value;
};

struct et131x_registers_txdma {
	// 0x1000 tx DMA control status register

	union et131x_txdma_ctrl {
		struct {
			uint32_t halt:1;
			uint32_t drop_TLP_disable:1;
			uint32_t unused2:2;
			uint32_t cache_thrshld:4;
			uint32_t sngl_epkt_mode:1;
			uint32_t traffic_class:4;
			uint32_t unused:19;
		};
		uint32_t value;
	} ctrl;

	// 0x1004 tx DMA packet ring base address

	uint32_t pr_base_hi;
	uint32_t pr_base_lo;

	// 0x100c tx DMA packet ring number of descriptors (10 bits)
	
	uint32_t pr_num_des;

	// 0x1010 tx DMA queue write address

	union et131x_wrapaddr10 txq_wr_addr;

	// 0x1014 tx DMA queue write address external

	union et131x_wrapaddr10 txq_wr_addr_ext;

	// 0x1018 tx DMA reqd queue address

	union et131x_wrapaddr10 txq_rd_addr;

	// 0x101c tx DMA writeback address
	
	uint32_t wb_addr_hi;
	uint32_t wb_addr_lo;

	// 0x1024 tx DMA service request
	
	union et131x_wrapaddr10 service_request;

	// 0x1028 tx DMA service complete
	
	union et131x_wrapaddr10 service_complete;

	// 0x102c tx DMA cache read index
	
	union et131x_wrapaddr4 cache_rd_index;

	// 0x1030 tx DMA cache write index
	
	union et131x_wrapaddr4 cache_wr_index;

	// 0x1034 tx DMA error register

	union et131x_txdma_error {
		struct {
			uint32_t pyld_resend:1;
			uint32_t pyld_rewind:1;
			uint32_t unused2:1;
			uint32_t unused3:1;
			uint32_t descr_resend:1;
			uint32_t descr_rewind:1;	
			uint32_t unused6:1;
			uint32_t unused7:1;
			uint32_t wrbk_resend:1;
			uint32_t wrbk_rewind:1;
			uint32_t unused:22;
		};
		uint32_t value;
	} error;

	// 0x1038 counters
	
	uint32_t desc_abort_cnt;
	uint32_t payload_abort_cnt;
	uint32_t writeBack_abort_cnt;
	uint32_t desc_timeout_cnt;
	uint32_t payload_timeout_cnt;
	uint32_t writeback_timeout_cnt;
	uint32_t desc_error_cnt;
	uint32_t payload_error_cnt;
	uint32_t writeback_error_cnt;
	uint32_t dropped_tlp_cnt;
	union et131x_wrapaddr10 new_service_complete;
	uint32_t ethernet_packet_cnt;
};

struct et131x_registers_rxdma {
	// 0x2000 rx DMA control status register
	
	union et131x_rxdma_ctrl {
		struct {
			uint32_t halt:1;
			uint32_t tc:3;
			uint32_t fbr_big_endian:1;
			uint32_t psr_big_endian:1;
			uint32_t pkt_big_endian:1;
			uint32_t dma_big_endian:1;
			uint32_t fbr0_size:2; // 0=128, 1=256, 2=512, 3=1024
			uint32_t fbr0_enable:1;
			uint32_t fbr1_size:2; // 0=2048, 1=4096, 2=8192, 3=16384
			uint32_t fbr1_enable:1;
			uint32_t unused14:1;
			uint32_t pkt_drop_disable:1;
			uint32_t pkt_done_flush:1;
			uint32_t halt_status:1;
			uint32_t unused:14;
		};
		uint32_t value;
	} ctrl;

	// 0x2004 rx DMA writeback address
	
	uint32_t wb_addr_lo;
	uint32_t wb_addr_hi;

	// 0x200c rx DMA number of packets done (8 bits)
	
	uint32_t num_pkt_done;

	// 0x2010 rx DMA max packet time (18 bits);
	
	uint32_t max_pkt_time;

	// 0x2014 rx DMA queue read address
	
	union et131x_wrapaddr10 rxq_rd_addr;

	// 0x2018 rx DMA queue read address external
	
	union et131x_wrapaddr10 rxq_rd_addr_ext;

	// 0x201c rx DMA queue write address

	union et131x_wrapaddr10 rxq_wr_addr;

	// 0x2020 rx DMA packet status register base
	
	uint32_t psr_base_lo;
	uint32_t psr_base_hi;

	// 0x2028 rx DMA packet status register number of descriptors (12 bits)
	
	uint32_t psr_num_des;

	// 0x202c rx DMA packet status ring available offset

	union et131x_wrapaddr12 psr_avail_offset;

	// 0x2030 rx DMA packet status ring full offset

	union et131x_wrapaddr12 psr_full_offset;
	
	// 0x2034 rx DMA packet status ring access index (5 bits)
	
	uint32_t psr_access_index;

	// 0x2038 rx DMA packet status ring minimum descriptors (12 bits)
	
	uint32_t psr_min_des;

	// 0x203c rx DMA free buffer ring 0 base address
	
	uint32_t fbr0_base_lo;
	uint32_t fbr0_base_hi;

	// 0x2044 rx DMA free buffer ring 0 number of descriptors (10 bits)
	
	uint32_t fbr0_num_des;

	// 0x2048 rx DMA free buffer ring 0 available offset
	
	union et131x_wrapaddr10 fbr0_avail_offset;

	// 0x204c rx DMA free buffer ring 0 full offset
	
	union et131x_wrapaddr10 fbr0_full_offset;

	// 0x2050 rx DMA free buffer ring 0 read index (5 bits)
	
	uint32_t fbr0_rd_index;

	// 0x2054 rx DMA free buffer ring 0 minimum descriptors (10 bits)
	
	uint32_t fbr0_min_des;

	// 0x2058 rx DMA free buffer ring 1 base address
	
	uint32_t fbr1_base_lo;
	uint32_t fbr1_base_hi;

	// 0x2060 rx DMA free buffer ring 1 number of descriptors (10 bits)
	
	uint32_t fbr1_num_des;

	// 0x2064 rx DMA free buffer ring 1 available offset
	
	union et131x_wrapaddr10 fbr1_avail_offset;

	// 0x2068 rx DMA free buffer ring 1 full offset
	
	union et131x_wrapaddr10 fbr1_full_offset;

	// 0x206c rx DMA free buffer ring 1 read index (5 bits)
	
	uint32_t fbr1_rd_index;

	// 0x2070 rx DMA free buffer ring 1 minimum descriptors (10 bits)
	
	uint32_t fbr1_min_des;
};

struct et131x_registers_txmac {
	// 0x3000 tx MAC control status register
	
	union et131x_txmac_ctrl {
		struct {
			uint32_t enable:1;
			uint32_t mif_disable:1;
			uint32_t mcif_disable:1;
			uint32_t fc_disable:1;
			uint32_t async_disable:1;
			uint32_t cksegnum:1;
			uint32_t ckbcnt_disable:1;
			uint32_t cklseg_disable:1;
			uint32_t unused:24;
		};
		uint32_t value;
	} ctrl;

	// 0x3004 tx MAC shadow pointer
	
	union et131x_txmac_shadow_ptr {
		struct {
			uint16_t txq_wr_ptr; // 11 bits
			uint16_t txq_rd_ptr; // 11 bits
		};
		uint32_t value;
	} shadow_ptr;

	// 0x3008 tx MAC error count
	
	union et131x_txmacc_err_cnt {
		struct {
			uint32_t fifo_underrun:4;
			uint32_t txq_underrun:4;
			uint32_t unused:24;
		};
		uint32_t value;
	} err_cnt;

	// 0x300c tx MAC max fill (12 bits)

	uint32_t max_fill;

	// 0x3010 tx MAC cf parameter

	union et131x_txmac_cf_param {
		struct {
			uint16_t cfpt;
			uint16_t cfep;
		};
		uint32_t value;
	} cf_param;

	// 0x3014 tx MAC test

	union et131x_txmac_txtest {
		struct {
			uint32_t txqtest_ptr:11;
			uint32_t unused11:1;
			uint32_t unused12:1;
			uint32_t unused13:1;
			uint32_t unused14:1;
			uint32_t enable:1;
			uint32_t unused:16;
		};
		uint32_t value;
	} txtest;

	// 0x3018 tx MAC error register
	
	union et131x_txmac_err {
		struct {
			uint32_t seg0:1;
			uint32_t segnum:1;
			uint32_t lseg:1;
			uint32_t bcnt:1;
			uint32_t txq_underrun:1;
			uint32_t ctrl2:1;
			uint32_t unused6:1;
			uint32_t unused7:1;
			uint32_t fifo_underrun:1;
			uint32_t unused:23;
		};
		uint32_t value;
	} err;

	// 0x301c tx MAC error interrupt
	
	union et131x_txmac_err err_int;

	// 0x3020 tx MAC bp control

	union et131x_txmac_bp_ctrl {
		struct {
			uint32_t xonxoff:1;
			uint32_t req:1;
			uint32_t unused:30;
		};
		uint32_t value;
	} bp_ctrl;
};

struct et131x_registers_rxmac {
	// 0x4000 rx MAC control status register
	
	union et131x_rxmac_ctrl {
		struct {
			uint32_t enable:1;
			uint32_t mcif_disable:1;
			uint32_t pkt_filter_disable:1;
			uint32_t wol_disable:1;
			uint32_t mif_disable:1;
			uint32_t async_disable:1;
			uint32_t rxmac_int_disable:1;
			uint32_t unused:25;
		};
		uint32_t value;
	} ctrl;

	// 0x4004 rx MAC Wake On Lan / CRC control

	union et131x_rxmac_wol_crc {
		struct {
			uint16_t valid_crc0:1;
			uint16_t valid_crc1:1;
			uint16_t valid_crc2:1;
			uint16_t valid_crc3:1;
			uint16_t valid_crc4:1;
			uint16_t ignore_broadcast:1;
			uint16_t ignore_multicast:1;
			uint16_t ignore_unicast:1;
			uint16_t ignore_link_change:1;
			uint16_t clear_interrupt:1;
			uint16_t ignore_mp:1;
			uint16_t ignore_pp:1;
			uint16_t unused:4;
		};
		uint32_t value;
	} wol_crc;

	// 0x4006 rx MAC CRC registers

	uint16_t crc0;
	uint16_t crc1;
	uint16_t crc2;
	uint16_t crc3;
	uint16_t crc4;

	// 0x4010 rx MAC Wake On Lan source address

	uint8_t wol_sa[6];
	uint8_t wol_sa_unused[2];

	// 0x4018 rx MAC Wake On Lan mask

	uint32_t wol_mask0_0;
	uint32_t wol_mask0_1;
	uint32_t wol_mask0_2;
	uint32_t wol_mask0_3;
	uint32_t wol_mask1_0;
	uint32_t wol_mask1_1;
	uint32_t wol_mask1_2;
	uint32_t wol_mask1_3;
	uint32_t wol_mask2_0;
	uint32_t wol_mask2_1;
	uint32_t wol_mask2_2;
	uint32_t wol_mask2_3;
	uint32_t wol_mask3_0;
	uint32_t wol_mask3_1;
	uint32_t wol_mask3_2;
	uint32_t wol_mask3_3;

	// 0x4068 rx MAC unicast packet filter address
	
	uint32_t uni_pf_addr1_hi;
	uint32_t uni_pf_addr2_hi;
	union et131x_rxmac_uni_pf_addr_lo {
		struct {
			uint16_t addr1_lo;
			uint16_t addr2_lo;
		};
		uint32_t value;
	} uni_pf_addr_lo;

	// 0x4074 rx MAC multicast hash
	
	uint32_t multi_hash1;
	uint32_t multi_hash2;
	uint32_t multi_hash3;
	uint32_t multi_hash4;

	// 0x4084 rx MAC packet filter control

	union et131x_rxmac_pf_ctrl {
		struct {
			uint16_t filter_broadcast:1;
			uint16_t filter_multicast:1;
			uint16_t filter_unicast:1;
			uint16_t filter_fragments:1;
			uint16_t unused:12;
			uint16_t min_pkt_size; // 7 bits
		};
		uint32_t value;
	} pf_ctrl;

	// 0x4088 r MAC memory controller interface control max segment

	union et131x_rxmac_mcif_ctrl_max_seg {
		struct {
			uint32_t seg_enable:1;
			uint32_t fc_enable:1;
			uint32_t max_size:8;
			uint32_t unused:22;
		};
		uint32_t value;
	} mcif_ctrl_max_seg;

	// 0x408c rx MAC memory controller interface water mark
	
	union et131x_rxmac_mcif_water_mark {
		struct {
			uint16_t mark_lo; // 10 bits
			uint16_t mark_hi; // 10 bits
		};
		uint32_t value;
	} mcif_water_mark;

	// 0x4090 rx MAC queue diag
	
	union et131x_rxmac_rxq_diag {
		struct {
			uint16_t wr_ptr; // 10 bits
			uint16_t rd_ptr; // 10 bits
		};
		uint32_t value;
	} rxq_diag;

	// 0x4094 rx MAC space available
	
	union et131x_rxmac_space_avail {
		struct {
			uint16_t space_avail; // 10 bits
			uint16_t space_avail_enable; // 1 bit
		};
		uint32_t value;
	} space_avail;

	// 0x4098 rx MAC management interface control
	
	union et131x_rxmac_mif_ctrl {
		struct {
			uint32_t drop_pkt_mask:17;
			uint32_t drop_pkt_enable:1;
			uint32_t unused:14;
		};
		uint32_t value;
	} mif_ctrl;

	// 0x409c rx MAC error register
	
	union et131x_rxmac_error {
		struct {
			uint32_t mcif:1;
			uint32_t pkt_filter:1;
			uint32_t async:1;
			uint32_t mif:1;
			uint32_t unused:28;
		};
		uint32_t value;
	} error;
};

struct et131x_registers_mac {
	// 0x5000 MAC control status registers
	
	union et131x_mac_ctrl1 {
		struct {
			uint32_t tx_enable:1;
			uint32_t syncd_tx_enable:1;
			uint32_t rx_enable:1;
			uint32_t syncd_rx_enable:1;
			uint32_t tx_flow:1;
			uint32_t rx_flow:1;
			uint32_t unused6:1;
			uint32_t unused7:1;
			uint32_t loop_back:1;
			uint32_t unused9_15:7;
			uint32_t reset_tx_fun:1;
			uint32_t reset_rx_fun:1;
			uint32_t reset_tx_mc:1;
			uint32_t reset_rx_mc:1;
			uint32_t unused20_29:10;
			uint32_t sim_reset:1;
			uint32_t soft_reset:1;
		};
		uint32_t value;
	} ctrl1;

	union et131x_mac_ctrl2 {
		struct {
			uint32_t full_duplex:1;
			uint32_t crc_enable:1;
			uint32_t pad_crc:1;
			uint32_t undefined:1;
			uint32_t len_check:1;
			uint32_t huge_frame:1;
			uint32_t unused6:1;
			uint32_t unused7:1;
			uint32_t if_mode:2;
			uint32_t unused10:1;
			uint32_t unused11:1;
			uint32_t preamble_len:4;
			uint32_t unused:16;
		};
		uint32_t value;
	} ctrl2;

	// 0x5008 MAC inter packet gap

	union et131x_mac_ipg {
		struct {
			uint8_t b2b; // 7 bits
			uint8_t min_enforce; // 8 bits
			uint8_t non_b2b_2; // 7 bits
			uint8_t non_b2b_1; // 7 bits
		};
		uint32_t value;
	} ipg;

	// 0x500c MAC half duplex
	
	union et131x_mac_hfdp {
		struct {
			uint32_t coll_window:10;
			uint32_t unused10:1;
			uint32_t unused11:1;
			uint32_t rexmit_max:4;
			uint32_t excess_defer:1;
			uint32_t no_backoff:1;
			uint32_t bp_no_backoff:1;
			uint32_t alt_beb_enable:1;
			uint32_t alt_beb_trunc:4;
			uint32_t unused:8;
		};
		uint32_t value;
	} hfdp;

	// 0x5010 MAC maximum frame length

	uint32_t max_fm_len; // 16 bits

	// 0x5014 MAC reserve registers
	
	uint32_t rsv1;
	uint32_t rsv2;

	// 0x501c MAC test
	
	uint32_t test; // 3 bits

	// 0x5020 MII management configuration
	
	union et131x_mac_mii_mgmt_cfg {
		struct {
			uint32_t clk_reset:3;
			uint32_t unused3:1;
			uint32_t preamble_suppress:1;
			uint32_t scan_auto_incremt:1;
			uint32_t unused:25;
			uint32_t reset:1;
		};
		uint32_t value;
	} mii_mgmt_cfg;

	// 0x5024 MII manamement command
	
	union et131x_mac_mii_mgmt_cmd {
		struct {
			uint32_t read_cycle:1;
			uint32_t scan_cycle:1;
			uint32_t unused:30;
		};
		uint32_t value;
	} mii_mgmt_cmd;

	// 0x5028 MII management address

	union et131x_mac_mii_mgmt_addr {
		struct {
			uint32_t reg:5;
			uint32_t unused5:1;
			uint32_t unused6:1;
			uint32_t unused7:1;
			uint32_t phy:5;
			uint32_t unused:19;
		};
		uint32_t value;
	} mii_mgmt_addr;

	// 0x502c MII management control

	union et131x_mac_mii_mgmt_ctrl {
		struct {
			uint16_t phy;
			uint16_t unused;
		};
		uint32_t value;
	} mii_mgmt_ctrl;

	// 0x5030 MII management status
	
	union et131x_mac_mii_mgmt_stat {
		struct {
			uint16_t phy;
			uint16_t unused;
		};
		uint32_t value;
	} mii_mgmt_stat;

	// 0x5034 MII management indicators
	
	union et131x_mac_mii_mgmt_indicator {
		struct {
			uint32_t busy:1;
			uint32_t scanning:1;
			uint32_t not_valid:1;
			uint32_t unused:29;
		};
		uint32_t value;
	} mii_mgmt_indicator;

	// 0x5038 MAC interface control
	
	union et131x_mac_if_ctrl {
		struct {
			uint32_t jab_protect:1;
			uint32_t unused1_6:6;
			uint32_t reset_gpsi:1;
			uint32_t disable_link_fail:1;
			uint32_t no_cipher:1;
			uint32_t force_quiet:1;
			uint32_t unused11_14:4;
			uint32_t reset_pe100x:1;
			uint32_t speed:1;
			uint32_t unused17_22:6;
			uint32_t reset_per_mii:1;
			uint32_t phy_mode:1;
			uint32_t lhd_mode:1;
			uint32_t ghd_mode:1;
			uint32_t tbi_mode:1;
			uint32_t unused28_30:3;
			uint32_t reset_if_module:1;
		};
		uint32_t value;
	} if_ctrl;

	// 0x503c MAC interface status
	
	union et131x_mac_if_stat {
		struct {
			uint32_t pe10t_jabber:1;
			uint32_t pe10t_sqe_error:1;
			uint32_t pe10t_loss_carrie:1;
			uint32_t pe100x_link_fail:1;
			uint32_t phy_speed:1;
			uint32_t phy_full_duplex:1;
			uint32_t phy_link_ok:1;
			uint32_t phy_jabber:1;
			uint32_t clash:1;
			uint32_t excess_defer:1;
			uint32_t unused:22;
		};
		uint32_t value;
	} if_stat;

	// 0x5040 MAC station address

	union et131x_mac_station_addr1 {
		struct {
			uint8_t octet3;
			uint8_t octet4;
			uint8_t octet5;
			uint8_t octet6;
		};
		uint32_t value;
	} station_addr1;

	union et131x_mac_station_addr2 {
		struct {
			uint8_t unused0;
			uint8_t unused1;
			uint8_t octet1;
			uint8_t octet2;
		};
		uint32_t value;
	} station_addr2;
};

struct et131x_registers_mac_stat {
	// 0x6000..0x607c unused
	
	uint32_t unused[32];

	//Tx/Rx 0-64 Byte Frame Counter
	uint32_t tr64;	//  0x6080

	//Tx/Rx 65-127 Byte Frame Counter
	uint32_t tr127;	//  0x6084

	//Tx/Rx 128-255 Byte Frame Counter
	uint32_t tr255;	//  0x6088

	//Tx/Rx 256-511 Byte Frame Counter
	uint32_t tr511;	//  0x608C

	//Tx/Rx 512-1023 Byte Frame Counter
	uint32_t tr1k;	//  0x6090

	//Tx/Rx 1024-1518 Byte Frame Counter
	uint32_t trmax;	//  0x6094

	//Tx/Rx 1519-1522 Byte Good VLAN Frame Count
	uint32_t trmgv;	//  0x6098

	//Rx Byte Counter
	uint32_t rbyt;	//  0x609C

	//Rx Packet Counter
	uint32_t rpkt;	//  0x60A0

	//Rx FCS Error Counter
	uint32_t rfcs;	//  0x60A4

	//Rx Multicast Packet Counter
	uint32_t rmca;	//  0x60A8

	//Rx Broadcast Packet Counter
	uint32_t rbca;	//  0x60AC

	//Rx Control Frame Packet Counter
	uint32_t rxcf;	//  0x60B0

	//Rx Pause Frame Packet Counter
	uint32_t rxpf;	//  0x60B4

	//Rx Unknown OP Code Counter
	uint32_t rxuo;	//  0x60B8

	//Rx Alignment Error Counter
	uint32_t raln;	//  0x60BC

	//Rx Frame Length Error Counter
	uint32_t rflr;	//  0x60C0

	//Rx Code Error Counter
	uint32_t rcde;	//  0x60C4

	//Rx Carrier Sense Error Counter
	uint32_t rcse;	//  0x60C8

	//Rx Undersize Packet Counter
	uint32_t rund;	//  0x60CC

	//Rx Oversize Packet Counter
	uint32_t rovr;	//  0x60D0

	//Rx Fragment Counter
	uint32_t rfrg;	//  0x60D4

	//Rx Jabber Counter
	uint32_t rjbr;	//  0x60D8

	//Rx Drop
	uint32_t rdrp;	//  0x60DC

	//Tx Byte Counter
	uint32_t tbyt;	//  0x60E0

	//Tx Packet Counter
	uint32_t tpkt;	//  0x60E4

	//Tx Multicast Packet Counter
	uint32_t tmca;	//  0x60E8

	//Tx Broadcast Packet Counter
	uint32_t tbca;	//  0x60EC

	//Tx Pause Control Frame Counter
	uint32_t txpf;	//  0x60F0

	//Tx Deferral Packet Counter
	uint32_t tdfr;	//  0x60F4

	//Tx Excessive Deferral Packet Counter
	uint32_t tedf;	//  0x60F8

	//Tx Single Collision Packet Counter
	uint32_t tscl;	//  0x60FC

	//Tx Multiple Collision Packet Counter
	uint32_t tmcl;	//  0x6100

	//Tx_reg Late Collision Packet Counter
	uint32_t tlcl;	//  0x6104

	//Tx Excessive Collision Packet Counter
	uint32_t txcl;	//  0x6108

	//Tx Total Collision Packet Counter
	uint32_t tncl;	//  0x610C

	//Tx Pause Frame Honored Counter
	uint32_t tpfh;	//  0x6110

	//Tx Drop Frame Counter
	uint32_t tdrp;	//  0x6114

	//Tx Jabber Frame Counter
	uint32_t tjbr;	//  0x6118

	//Tx FCS Error Counter
	uint32_t tfcs;	//  0x611C

	//Tx Control Frame Counter
	uint32_t txcf;	//  0x6120

	//Tx Oversize Frame Counter
	uint32_t tovr;	//  0x6124

	//Tx Undersize Frame Counter
	uint32_t tund;	//  0x6128

	//Tx Fragments Frame Counter
	uint32_t tfrg;	//  0x612C

	// 0x6130 MAC stat registers
	
	union et131x_mac_stat_reg1 {
		struct {
			uint32_t rdrp:1;
			uint32_t rjbr:1;
			uint32_t rfrg:1;
			uint32_t rovr:1;
			uint32_t rund:1;
			uint32_t rcse:1;
			uint32_t rcde:1;
			uint32_t rflr:1;
			uint32_t raln:1;
			uint32_t rxuo:1;
			uint32_t rxpf:1;
			uint32_t rxcf:1;
			uint32_t rbca:1;
			uint32_t rmca:1;
			uint32_t rfcs:1;
			uint32_t rpkt:1;
			uint32_t rbyt:1;
			uint32_t unused17_24:8;
			uint32_t trmgv:1;
			uint32_t trmax:1;
			uint32_t tr1k:1;
			uint32_t tr511:1;
			uint32_t tr255:1;
			uint32_t tr127:1;
			uint32_t tr64:1;
		};
		uint32_t value;
	} carry1;

	union et131x_mac_stat_reg2 {
		struct {
			uint32_t tdrp:1;
			uint32_t tpfh:1;
			uint32_t tncl:1;
			uint32_t txcl:1;
			uint32_t tlcl:1;
			uint32_t tmcl:1;
			uint32_t tscl:1;
			uint32_t tedf:1;
			uint32_t tdfr:1;
			uint32_t txpf:1;
			uint32_t tbca:1;
			uint32_t tmca:1;
			uint32_t tpkt:1;
			uint32_t tbyt:1;
			uint32_t tfrg:1;
			uint32_t tund:1;
			uint32_t tovr:1;
			uint32_t txcf:1;
			uint32_t tfcs:1;
			uint32_t tjbr:1;
			uint32_t unused:12;
		};
		uint32_t value;
	} carry2;

	union et131x_mac_stat_reg1 carry1_mask;
	union et131x_mac_stat_reg2 carry2_mask;
};

struct et131x_registers_mmc {
	// 0x7000 mmc control status register

	union et131x_mmc_ctrl {
		struct {
			uint32_t enable:1;
			uint32_t arb_disable:1;
			uint32_t rxmac_disable:1;
			uint32_t txmac_disable:1;
			uint32_t txdma_disable:1;
			uint32_t rxdma_disable:1;
			uint32_t force_ce:1;
			uint32_t unused:25;
		};
		uint32_t value;
	} ctrl;

	// 0x7004 mmc sram access

	union et131x_mmc_sram_access {
		struct {
			uint32_t req_access:1;
			uint32_t wr_access:1;
			uint32_t is_ctrl_word:1;
			uint32_t unused3:1;
			uint32_t req_addr:10;
			uint32_t unused14:1;
			uint32_t unused15:1;
			uint32_t byte_enable:16;
		};
		uint32_t value;
	} sram_access;

	// 0x7008 mmc sram words

	uint32_t sram_word1;
	uint32_t sram_word2;
	uint32_t sram_word3;
	uint32_t sram_word4;
};

struct et131x_registers {
	struct et131x_registers_global global;
	char unused_global[0x1000 - sizeof(struct et131x_registers_global)];
	struct et131x_registers_txdma txdma;
	char unused_txdma[0x1000 - sizeof(struct et131x_registers_txdma)];
	struct et131x_registers_rxdma rxdma;
	char unused_rxdma[0x1000 - sizeof(struct et131x_registers_rxdma)];
	struct et131x_registers_txmac txmac;
	char unused_txmac[0x1000 - sizeof(struct et131x_registers_txmac)];
	struct et131x_registers_rxmac rxmac;
	char unused_rxmac[0x1000 - sizeof(struct et131x_registers_rxmac)];
	struct et131x_registers_mac mac;
	char unused_mac[0x1000 - sizeof(struct et131x_registers_mac)];
	struct et131x_registers_mac_stat stat;
	char unused_stat[0x1000 - sizeof(struct et131x_registers_mac_stat)];
	struct et131x_registers_mmc mmc;
	char unused_mmc[0x1000 - sizeof(struct et131x_registers_mmc)];
};

#endif
